/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __MATH_H
#define __MATH_H

#include <cstdlib>
#include <limits>
using namespace std;

#include "vec.h"

/**
@author Peter Müller
 */
 
const float Pi = 3.141592653589793238512808959406186204433f;

template <class T> inline T clamp(T x, T a, T b) {
	return min(max(x,a),b);
}

inline float deg2rad(float d) {
	return d * Pi / 180.0f;
}

inline float rad2deg(float r) {
	return r * 180.0f / Pi;
}

inline float frand() {
	return (float)(rand() % RAND_MAX) / RAND_MAX;
}

inline float frand(float min, float max) {
	return min + frand() * (max - min);
}

inline int rand(int min, int max) {
	return min + rand() % (min-max+1);
}

class Plane
{
public:
	Plane(const Vec3 & _NN, float _dd) : N(_NN), d(_dd) { }
	virtual ~Plane() { }
    
	Vec3 N;
	float d;
};

//bool InterDynamicSphereSphere(const Vec3 & c1, float r1, const Vec3 & W, const Vec3 & c2, float r2, float & time);
bool TestDynamicSpherePlane(const Vec3 & c1, float r1, const Vec3 & W, const Plane & plane, float & time);


#endif
