/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef COMETENSCENE_H
#define COMETENSCENE_H

#include <list>
#include <map>

using namespace std;

#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glu.h>

#include "math.h"
#include "particlesystem.h"
#include "effect.h"
#include "global.h"

#include "comet.h"

/**
@author Peter Müller
*/

class Comet;

enum _colors {
	C_RED,
	C_BLUE,
	C_GREEN,
	C_BUNT,
	C_EXTREMBUNT
};

class Timer {
public:
	Timer() {
		xtime = 0.0f;
		ytime = 0.0f;
		time_scale = 1.0f;
	}
	~Timer() {}

	void setTime(float start_time, float real_time) {
		ytime = start_time;
		xtime = real_time;
	}

	void setScale(float ts, float real_time) {
		ytime = getTime(real_time);
		xtime = real_time;
		time_scale = ts;
	}

	float getTime(float real_time) {
		return (real_time - xtime) * time_scale + ytime;
	}
	
private:
	float xtime;
	float ytime;
	float time_scale;	
};

class CometenScene {
public:
	CometenScene() : box_size(150), fov(70.0f){};
	~CometenScene() { quit(); };
	
	void init(float time, GLuint bg, int _colors, float _create_intervall, 
	          float _lightmap_alpha, bool _do_matrix, bool _do_slowmotion, bool _do_blitz);
    void resize(int res_x, int res_y);
    void quit();
	void process(float time);
	void render();
	
	void addComet(Comet * comet);
	void addEffect(Effect * effect);
	
	GLuint getTexture(const string & file);
	const Vec3 & getCamPos() { return campos; }
	
	float getBoxSize() { return box_size; }
	float getBoxHeight() { return box_height; }
		
private:
	const float box_size;
	const float fov;
	float box_cam_distance;
	float box_height;
	
	Vec3 campos;
	Vec3 cam_viewpoint;
	float fade; 		// for fade to black
	bool black_bars;
	
	// every comet or effect musst be in scenetime
	Timer scenetimer;
	
	GLuint background, lightmap;
	float lightmap_alpha;
	int colors;
	map<string, GLuint> textures;
	
	list<Plane> planes;
	list<Comet*> comets;
	list<Effect*> effects;
	
	enum _state {
		S_NORMAL,
		S_MATRIX,
		S_SLOWMOTION
	};
	
	int state;
	float state_start;
	
	bool do_matrix;
	bool do_slowmotion;
	bool do_blitz;
	
	bool blitz;
	float last_blitz;
	
	// in scene time
	float last_comet_process;
	float next_comet;
	float create_intervall;
	float next_blitz_effect;
	// in real time	
	float next_cam_effect;
	
	// in real time
	void processNormal(float time);
	void processMatrix(float time);
	void processSlowmotion(float time);
	// in scene time
	void processBlitz(float time);
	void processScene(float time);
	void processComets(float time);
	void processEffects(float time);
	// scene time
	void collision(float time, float deltat);
	void cometHitPlane(float time, Comet * comet, const Plane& plane);
	
	void renderBackground();
	
	void loadTexture(const string & file);
	void freeTexture(const string & file);
	void freeTextures();
};



#endif
