/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "lightning.h"

Lightning::Lightning(GLuint _texture, const Vec3 & _start, const Vec3 & _end, 
					 float _width, int steps, int num_lightnings, float time, 
					 float life_time, float _switch_time, float _flacker_time) 
:
Effect(),
texture(_texture),
start(_start),
end(_end),
width(_width),
switch_time(_switch_time),
flacker_time(_flacker_time),
vertices(NULL)
{
	vector<Vec3> point_list;
	vector<Vec3>::iterator i;
	int j,k;
	
	end_time = time + life_time;
	switch_time = _switch_time;
	last_switch = time;
	current_lightning = 0;
	visible = true;
	
	lightnings.clear();
	max_vertices = 0;
		
	for (k = 0; k < num_lightnings; k++) {
			
		lightnings.push_back(vector<Line>());
		vector<Line> & li = *lightnings.rbegin();
			
		li.clear();			
		createLightning(start, end, steps, li);
		
		max_vertices = max(max_vertices, (unsigned int)li.size() * 12); 
	}
		
	vertices = new LightningVertex[max_vertices];
}

Lightning::~Lightning() {
	delete[] vertices;
}

void Lightning::process(float time) {
	last_process_time = time;
	
	if (time < last_switch + switch_time) {
		visible = true;
	}
	else if (time < last_switch + switch_time + flacker_time) {
		visible = false;
	}
	else {
		current_lightning = rand() % lightnings.size();
		last_switch = time;
		visible = true;
	}
}

int Lightning::buildVertexArray(const vector<Line>& li, const Vec3& campos) {
	Vec3 dir, up, right;
	int num_vertices;
	LightningVertex * v;
	vector<Line>::const_iterator i;
	
	// line to cam direction
	dir = campos - ((start + end) / 2);
	dir.normalize();
	
	v = vertices;
	num_vertices = 0;
	
	for (i = li.begin(); i != li.end(); i++) {
		const Line & line = *i;
		
		right = line.dir;
		up = dir.cross(right);
		
		v->pos = line.p1 - right * (width/2) - up * (width/2);
		v->uv.x = 0.0f;
		v->uv.y = 0.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p1 - right * (width/2) + up * (width/2);
		v->uv.x = 0.0f;
		v->uv.y = 1.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p1 + up * (width/2);
		v->uv.x = 0.5f;
		v->uv.y = 1.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p1 - up * (width/2);
		v->uv.x = 0.5f;
		v->uv.y = 0.0f;
		
		v++;
		num_vertices++;	
		
		////	
		
		v->pos = line.p1 - up * (width/2);
		v->uv.x = 0.49f;
		v->uv.y = 0.0f;
		
		v++;
		num_vertices++;		
		
		v->pos = line.p1 + up * (width/2);
		v->uv.x = 0.49f;
		v->uv.y = 1.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p2 + up * (width/2);
		v->uv.x = 0.51f;
		v->uv.y = 1.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p2 - up * (width/2);
		v->uv.x = 0.51f;
		v->uv.y = 0.0f;
		
		v++;
		num_vertices++;	
		
		////
		v->pos = line.p2 - up * (width/2);
		v->uv.x = 0.5f;
		v->uv.y = 0.0f;
		
		v++;
		num_vertices++;	
		
		v->pos = line.p2 + up * (width/2);
		v->uv.x = 0.5f;
		v->uv.y = 1.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p2 + up * (width/2) + right * (width/2);
		v->uv.x = 1.0f;
		v->uv.y = 1.0f;
		
		v++;
		num_vertices++;
		
		v->pos = line.p2 - up * (width/2) + right * (width/2);
		v->uv.x = 1.0f;
		v->uv.y = 0.0f;
		
		v++;
		num_vertices++;
	}
	
	return num_vertices;
}

void Lightning::render(const Vec3 & campos) {
	int num_vertices;
	
	if (!visible) return;
	
	vector<Line>& li = lightnings[current_lightning];
	num_vertices = buildVertexArray(li, campos);
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glVertexPointer(3, GL_FLOAT, sizeof(LightningVertex), &vertices[0].pos);
	glTexCoordPointer(2, GL_FLOAT, sizeof(LightningVertex), &vertices[0].uv);
	glLineWidth(2.0f);
	
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	
	glDrawArrays(GL_QUADS, 0, num_vertices);
	
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}

void Lightning::stop() {
}

bool Lightning::done() {
	return last_process_time > end_time;
}

// create lightning between start and end
// steps = 0 <=> direct line

void Lightning::createLightning(const Vec3 & start, const Vec3 & end, 
								int steps, vector<Line> & lines) 
{
	Vec3 cur, last;
	Vec3 diff = end - start;
	float l = diff.getLength();
	Vec3 dir = diff / l; 
	Line line;
	float maxvar = l / (25+1);
	float minvar = 0;
	
	cur = start;
	steps = max(0,steps);
	
	for (int i = 0; i < steps; i++) {
		last = cur;
		cur = start + (diff / (steps+1) * i) + (Vec3::randomUnit() * frand(minvar, maxvar));
		
		line.p1 = last;
		line.p2 = cur;
		line.dir = cur - last;
		line.l = line.dir.getLength();
		
		if (line.l != 0.0f) {
			line.dir /= line.l;
		}
		else {
			line.dir = Vec3::X;
		}
		
		lines.push_back(line);
	}
	
	line.p1 = cur;
	line.p2 = end;
	line.dir = end - cur;
	line.l = line.dir.getLength();
		
	if (line.l != 0.0f) {
		line.dir /= line.l;
	}
	else {
		line.dir = Vec3::X;
	}
	
	lines.push_back(line);
}

