/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIGHTNING_H
#define LIGHTNING_H

#include <vector>
using namespace std;

#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glu.h>

#include "cometen_math.h"
#include "effect.h"

/**
@author Peter Müller
*/
class Lightning : public Effect
{
public:
    Lightning(GLuint texture, const Vec3 & start, const Vec3 & end, 
			  float width, int steps, int num_lightnings, float time, 
			  float life_time, float switch_time, float flacker_time);
    ~Lightning();
	
	virtual void process(float time);
	virtual void render(const Vec3 & viewpoint);
	virtual bool done();
	virtual void stop();
	
private:
	class Line {
	public:
		Vec3 p1, p2;
		Vec3 dir; // normalize dir, (p2 - p1)
		float l;  // p1 + l*dir = p2
	};
	
	class LightningVertex {
	public:
		Vec3 pos;
		Vec2 uv;
	};
	
	void createLightning(const Vec3 & start, const Vec3 & end, 
						 int steps, vector<Line> & lines);
	int buildVertexArray(const vector<Line>& li, const Vec3& campos);
	
	GLuint texture;
	
	vector<vector<Line> > lightnings;
	
	LightningVertex *vertices;
	unsigned int max_vertices;
	
	Vec3 start, end;
	float width;
	float last_process_time;
	float end_time;
	float switch_time;
	float flacker_time;
	float last_switch;
	bool visible;
	int current_lightning;
};

#endif
